""" Configuration Specification (template) Module
    Creates the configSpec string for QCOBJ

    RV 20170906
"""
from __future__ import print_function
from collections import OrderedDict as Odict

# Local imports
from  qcobj.qconfigobj import makeSpec, MANY, reindent

#------------------------------------------------------------------------------
level = 0
# This is the root section
secname = ''

#------------------------------------------------------------------------------
subsection = Odict((
    # Key                 # [Description1], [Description2], units
    # units can be:
    #   any expression containing units as defined in pint module
    # OR
    # any of (boolean, int, float, string)
    # OR
    # a list of options
    # In case  of int and float optional minimum or min and max values
    # can be specified separated by ONE SINGLE blank char
     ('description', (
        '',
        'A descripion of this configuration file',
        '',
        'string',
        'A meaningful descripion of this configuration file')),
    ('voltage', (
        'The voltage applied',
        'V, 0, 100',
        13.8)),
    ('C', (
        'The capacitance of the capacitor',
        'F, 0, 1000',
        0.1)),
    ('UG', (
        'Universal gravity constant',
        'N * m**2 / kg**2',
        6.672e-11)),
    ('numberOfSteps', (
        'Maximum number of steps',
        'int, 0, 5',
        1)),
    ('myrange', (
        'The allowed range',
        'float, -5.5, 14.4',
        12.0)),
    ('cake', (
        'The cake that we will eat',
        "none Sacher Saint-Honore apple-pie".split(),
        'Sacher')),
    ('approximate', (
        'Use approximation',
        'boolean',
        False)),
    ('color_index', (
        'The color index',
        'dimensionless',
        1.0)),
    ('configFiles', (
        'List of more configuration files blank separated',
        'string',
        '')),
    ))
rootSpec = makeSpec(secname, subsection, level)
# Remove indentation
rootspec = "\n".join([line.lstrip() for line in rootSpec.splitlines()])

#------------------------------------------------------------------------------
level = 1
secname = 'Ingredients'
#------------------------------------------------------------------------------
subsection = Odict((
    # Key                 # [Description1], [Description2], units
    # units can be:
    #   any expression containing units as defined in pint module
    # OR
    # any of (boolean, int, float, string)
    # OR
    # a list of options
    # In case  of int and float optional minimum or min and max values
    # can be specified separated by ONE SINGLE blank char
    ('sugar', (
        'Enable sugar',
        'boolean',
        False)),
    ('fruits', (
        'A list of two fruits at your choice',
         'string_list 2 2',
        "apple, orange")),
    ('roomTemp', (
        'Room temperature',
        'degC, 18, 26',
        20.0)),
    ('fraction', (
        'Some decimal value (floats are welcome, as always)',
        'float, 0, 1',
        0.25)),
    ))
subspec = makeSpec(secname, subsection, level)

#------------------------------------------------------------------------------
level = 2
secname = 'Regions'
#------------------------------------------------------------------------------
subsection = Odict((
    ('enabled', (
        'Enable all regions',
        'boolean',
        False)),
    ))
subsubspec = makeSpec(secname, subsection, level)

level += 1
secname = MANY
subsection = Odict((
    # Key                 # [Description1], [Description2], units
    # units can be:
    #   any expression containing units as defined in pint module
    # OR
    # any of (boolean, int, float, string)
    # OR
    # a list of options
    # In case  of int and float optional minimum or min and max values
    # can be specified separated by ONE SINGLE blank char
    ('enabled', (
        'Enable this region',
        'boolean',
        False)),
    ('T', (
        'Constant Temperature',
        'degC',
        18.0)),
    ('polygons', (
        'Polygons defined by their vertices, one per line,',
        'separated by nl',
        'string',
        "")),
    ))
many = makeSpec(secname, subsection, level)

morespec = """
#
# More Sections like this can be added with different names
# (e.g. region1, region2, ...) and the same syntax:
#
"""

configSpec = '\n'.join((
    rootspec,
    subspec,
    subsubspec,
    reindent(morespec, level * 4),
    many,
    ))

if __name__ == '__main__':
    configspecFile = "configspec.cfg"
    with open(configspecFile, "w") as out:
        out.write(configSpec)
    print("% has been created" % configspecFile)
